2012-06-26 02:51

writeStreamingDo: block
	| mapWriter |
	mapWriter := NeoJSONStreamingWriter on: self.
	self indentedDo: [
		block value: mapWriter ].
	mapWriter wasUsed
		ifTrue: [ self newlineIndent ]
		ifFalse: [ self prettyPrintSpace ]!
writeNull
	writeStream nextPutAll: 'null'!   
on: aWriteStream
	"Initialize on aWriteStream, which should be a character stream that 
	implements #nextPut:, #nextPutAll:, #space and (optionally) #close."

	writeStream := aWriteStream
!
newObjectMappingFor: smalltalkClass
	| mapping |
	mapping := NeoJSONObjectMapping new.
	mapping subjectClass: smalltalkClass.
	self mappings at: smalltalkClass put: mapping.
	^ mapping!
resolveMappingFor: class
	(class respondsTo: #neoJsonMapping:)
		ifTrue: [ 
			class perform: #neoJsonMapping: with: self ]!   
neoJsonOn: neoJSONWriter
	neoJSONWriter writeString: self!   
subjectClass: smalltalkClass
	subjectClass := smalltalkClass!
newlineIndent
	prettyPrint
		ifTrue: [ 
			writeStream nextPut: Character cr.
			level timesRepeat: [ writeStream nextPut: Character tab ] ]! 
mappings
	^ mappings
		ifNil: [ mappings := Dictionary new ]!   
writeObject: anObject
	| mapping |
	mapping := self mappingFor: anObject class.
	mapping
		writeObject: anObject
		on: self! 
mapAccessors: accessors
	accessors do: [ :each |
		self mapAccessor: each ]!
mapInstVars: instVarNames
	instVarNames do: [ :each |
		self mapInstVar: each ]!
writeFloat: float
	float printOn: writeStream!   
writeKey: key value: value
	first 
		ifTrue: [ 
			jsonWriter newlineIndent. 
			first := false ]
		ifFalse: [ 
			jsonWriter mapElementSeparator ].
	jsonWriter encodeKey: key value: value ! 
writeElement: value
	first 
		ifTrue: [ 
			jsonWriter newlineIndent. 
			first := false ]
		ifFalse: [ 
			jsonWriter listElementSeparator ].
	jsonWriter nextPut: value !
newCustomMappingFor: schemaName
	| mapping |
	mapping := NeoJSONCustomMapping new.
	mapping schemaName: schemaName.
	self mappings at: schemaName put: mapping.
	^ mapping!  
mappingFor: schema ifAbsent: block
	(self mappings includesKey: schema)
		ifFalse: [ self resolveMappingFor: schema ].
	^ self mappings
		at: schema
		ifAbsent: block!  
stringStreamContents: block
	"Like String streamContents: block
	but reusing the underlying buffer for improved efficiency"
	
	stringStream 
		ifNil: [ 
			stringStream := (String new: 32) writeStream ].
	stringStream reset.
	block value: stringStream.
	^ stringStream contents!   
expectChar: character
	"Expect character and consume input and optional whitespace at the end,
	 throw an error otherwise."

	(self matchChar: character)
		ifFalse: [ self error: character asString, ' expected' ]!
schemaName: schema
	schemaName := schema!
writeMapStreamingDo: block
	writeStream nextPut: ${.
	self writeStreamingDo: block.
	writeStream nextPut: $}!  
on: aJSONWriter
	jsonWriter := aJSONWriter!  
NeoJSONMappingNotFound sllUnBind.
NeoJSONWriter sllUnBind.
NeoJSONMapper sllUnBind.
NeoJSONCustomMapping sllUnBind.
NeoJSONObjectMapping sllUnBind.
NeoJSONParseError sllUnBind.
NeoJSONPropertyMapping sllUnBind.
NeoJSONReader sllUnBind.
NeoJSONStreamingWriter sllUnBind.
!  
parseMapKeysDo: block
	self parseMapDo: [ | key |
		key := self parsePropertyName.
		self expectChar: $:.
		block value: key ]!   
parseListElementsDo: block
	self parseListDo: [ 
		block value: self parseValue ]!  
propertyName: string
	propertyName := string!
writeObject: anObject on: jsonWriter
	writer value: jsonWriter value: anObject!  
parseCharacter
	| char |
	(char := readStream next) = $\ 
		ifFalse: [ ^ char ].
	('/"\' includes: (char := readStream next))
		ifTrue: [ ^ char ].
	('bfnrt' includes: char)
		ifTrue: [ ^ self unescapeChar: char ].
	char = $u
		ifTrue: [ ^ self parseCharacterHex ].
	self error: 'invalid escape character \' , (String with: char)!  
mapInstVar: instVarName to: propertyName
	| index |
	index := subjectClass 
		instVarIndexFor: instVarName 
		ifAbsent: [ self error: 'Inst var not found: ', instVarName printString ].
	^ self 
		mapProperty: propertyName
		getter: [ :object | object instVarAt: index ]
		setter: [ :object :value | object instVarAt: index put: value ] ! 
parseString
	| result |
	self expectChar: $".
	result := self stringStreamContents: [ :stream |
		[ readStream atEnd or: [ readStream peek = $"] ] 
			whileFalse: [ 
				stream nextPut: self parseCharacter ] ].
	self expectChar: $".
	^ result!   
readFrom: jsonReader
	| anObject |
	anObject := subjectClass new.
	jsonReader parseMapKeysDo: [ :key |
		(self propertyNamed: key ifAbsent: [ nil ])
			ifNil: [
				"read, skip & ignore value"
				jsonReader next ]
			ifNotNil: [ :mapping |
				mapping readObject: anObject from: jsonReader ] ].
	^ anObject!
prettyPrint: boolean
	prettyPrint := boolean!
new

	^super new
		initialize;
		yourself.!   
parseValue
	| char |
	readStream atEnd ifFalse: [ 
		char := readStream peek.
		char = ${
			ifTrue: [ ^ self parseMap ].
		char = $[
			ifTrue: [ ^ self parseList ].
		char = $"
			ifTrue: [ ^ self parseString ].
		(char = $- or: [ char isDigit ])
			ifTrue: [ ^ self parseNumber ].
		self parseConstantDo: [ :value | ^ value ] ].
	self error: 'invalid input: ', char asString!   
parseMap
	| map |
	map := self mapClass new.
	self parseMapKeysAndValuesDo: [ :key :value |
		map at: key put: value ].
	^ map!  
escapeChar: code
	code = 34
		ifTrue: [ ^ writeStream nextPutAll: '\"' ].
	code = 92
		ifTrue: [ ^ writeStream nextPutAll: '\\' ].
	code = 47
		ifTrue: [ ^ writeStream nextPutAll: '\/' ].
	code = 8
		ifTrue: [ ^ writeStream nextPutAll: '\b' ].
	code = 12
		ifTrue: [ ^ writeStream nextPutAll: '\f' ].		
	code = 10
		ifTrue: [ ^ writeStream nextPutAll: '\n' ].		
	code = 13
		ifTrue: [ ^ writeStream nextPutAll: '\r' ].		
	code = 9
		ifTrue: [ ^ writeStream nextPutAll: '\t' ].		
	writeStream nextPutAll: '\u'.
	code printOn: writeStream base: 16 nDigits: 4!   
fromString: string as: schema
	^ (self on: string readStream) nextAs: schema!
initialize
	super initialize.
	first := true!   
mapAllInstVarsFor: smalltalkClass
	"Map the instance variables of smalltalkClass and all its super classes"

	| mapping |
	mapping := self newObjectMappingFor: smalltalkClass.
	mapping mapAllInstVars.
	^ mapping!
mapClass: collectionClass
	mapClass := collectionClass!  
for: schemaName customDo: block
	"Create and add a new custom mapping for schemaName.
	The code in block should further customize the mapping."
	
	| mapping |
	mapping := self newCustomMappingFor: schemaName.
	block value: mapping.
	^ mapping!
parseNumberFraction
	| number power |
	number := 0.
	power := 1.0.
	[ readStream atEnd not and: [ readStream peek isDigit ] ] 
		whileTrue: [
			number := 10 * number + readStream next digitValue.
			power := power * 10.0 ].
	^ number / power!   
mapAccessor: accessor mutator: mutator to: propertyName
	^ self 
		mapProperty: propertyName
		getter: [ :object | object perform: accessor ]
		setter: [ :object :value | object perform: mutator with: value ] !
mapInstVar: instVarName
	^ self mapInstVar: instVarName to: instVarName !
writeInteger: integer
	integer printOn: writeStream
	!  
next
	"Primary interface to parse a JSON value.
	Return either primitives, a listClass or a mapClass instance."
	
	^ self 
		consumeWhitespace; 
		parseValue!  
writeList: collection
	self writeListStreamingDo: [ :jsonListWriter |
		collection do: [ :each |
			jsonListWriter writeElement: each ] ]! 
parseList
	^ self listClass 
		streamContents: [ :stream |
			self parseListElementsDo: [ :each |
				stream nextPut: each ] ]

		! 
initialize
	super initialize.
	properties := Dictionary new.!   
parseListDo: block
	self expectChar: $[.
	(self matchChar: $]) 
		ifTrue: [ ^ self ].
	[ readStream atEnd ] 
		whileFalse: [
			block value.
			(self matchChar: $]) 
				ifTrue: [ ^ self ].
			self expectChar: $, ].
	self error: 'end of list expected'!
unescapeChar: char
	"Handle one of the named character escapes, b, f, n, r or t. Else fail."
	
	char = $b
		ifTrue: [ ^ Character backspace ].
	char = $f
		ifTrue: [ ^ Character newPage ].
	char = $n
		ifTrue: [ ^ Character lf ].
	char = $r
		ifTrue: [ ^ Character cr ].
	char = $t
		ifTrue: [ ^ Character tab ].
	self error: 'Unknown escape char: ', (String with: char)!  
parseMapKeysAndValuesDo: block
	self parseMapKeysDo: [ :key |
		block value: key value: self parseValue ]!  
mapAllInstVars
	self mapInstVars: subjectClass allInstVarNames!  
parseCharacterHex
	| value |
	value := self parseCharacterHexDigit.
	3 timesRepeat: [ 
		value := (value << 4) + self parseCharacterHexDigit ].
	^ Character codePoint: value!   
parseNumberInteger
	| number |
	number := nil.
	[ readStream atEnd not and: [ readStream peek isDigit ] ] 
		whileTrue: [ 
			number := 10 * (number ifNil: [ 0 ]) + readStream next digitValue ].
	number ifNil: [ self error: 'Integer digit expected' ].
	^ number! 
mapAccessor: accessor
	^ self mapAccessor: accessor to: accessor !   
new

	^super new
		initialize;
		yourself.!   
on: writeStream
	"Initialize on writeStream, which should be a character stream that 
	implements #nextPut:, #nextPutAll:, #space and (optionally) #close."

	^ self new
		on: writeStream;
		yourself! 
mapInstVarsFor: smalltalkClass
	"Map the instance variables of smalltalkClass"

	| mapping |
	mapping := self newObjectMappingFor: smalltalkClass.
	mapping mapInstVars.
	^ mapping!
readFrom: jsonReader
	^ reader value: jsonReader!
close
	readStream ifNotNil: [
		readStream close.
		readStream := nil ]!   
fromString: string
	^ (self on: string readStream) next! 
new

	^super new
		initialize;
		yourself.!   
for: smalltalkClass do: block
	"Create and add a new standard object mapping for smalltalkClass.
	The code in block should further customize the mapping."
	
	| mapping |
	mapping := self newObjectMappingFor: smalltalkClass.
	block value: mapping.
	^ mapping! 
nextPut: anObject
	anObject neoJsonOn: self! 
readObject: anObject from: jsonReader
	| value |
	value := jsonReader nextAs: valueSchema.
	setter value: anObject value: value!   
error: text
	NeoJSONParseError signal: text! 
standardMessageText
	^ String streamContents: [ :stream |
		stream << 'No mapping found for '.
		stream print: self object.
		stream << ' in '.
		stream print: self collection class ]! 
mapProperty: propertyName getter: readBlock setter: writeBlock
	| propertyMapping |
	propertyMapping := NeoJSONPropertyMapping new.
	propertyMapping 
		propertyName: propertyName;
		getter: readBlock;
		setter: writeBlock.
	properties at: propertyName put: propertyMapping.
	^ propertyMapping !
match: string do: block
	"Try to read and consume string and execute block if successful.
	Else do nothing (but do not back up)"
	 
	(string allSatisfy: [ :each | self matchChar: each ])
		ifTrue: block
!
mapClass
	"Return the collection type to use for JSON maps"
	
	^ mapClass ifNil: [ Dictionary ]!   
parseConstantDo: block
	"Parse and consume either true|false|nil|null and execute block 
	or else do nothing (but do not back up).
	Hand written implementation to avoid the use of #position:"
	
	(readStream peek = $t)
		ifTrue: [
			^ self match: 'true' do: [ block value: true ] ].
	(readStream peek = $f)
		ifTrue: [
			^ self match: 'false' do: [ block value: false ] ].
	(readStream peek = $n)
		ifTrue: [
			^ self match: 'null' do: [ block value: nil ] ]
!  
initialize
	super initialize.
	prettyPrint := false.
	level := 0!  
printOn: stream
	super printOn: stream.
	stream nextPut: $(; print: propertyName; nextPut: $)!  
setter: block
	setter := block!  
reader: block
	"Set the reader to block, a 1 argument block that when given a jsonReader 
	as argument reads the object that I represent"
	
	reader := block! 
propertyNamesAsSymbols: boolean
	propertyNamesAsSymbols := boolean!  
nextAs: schema
	"Secondary interface to parse JSON.
	Return objects according to schema."
	
	| mapping |
	schema ifNil: [ ^ self next ].
	self consumeWhitespace.
	mapping := self mappingFor: schema.
	^ mapping readFrom: self! 
toString: object
	^ String streamContents: [ :stream |
			(self on: stream) nextPut: object ]!  
listElementSeparator
	writeStream nextPut: $,.
	self newlineIndent
!   
matchChar: character
	"Tries to match character, consume input and 
	answer true if successful and consumes whitespace at the end.
	When character does not match, return false and leave stream untouched."
	
	^ readStream peek = character
		ifTrue: [
			readStream next.
			self consumeWhitespace.
			true ]
		ifFalse: [
			false ]!
mapAccessor: accessor to: propertyName
	^ self 
		mapProperty: propertyName
		getter: [ :object | object perform: accessor ]
		setter: [ :object :value | object perform: accessor asMutator with: value ] !  
wasUsed
	^ first not!
parsePropertyName
	| name |
	name := self parseValue.
	^ name isString
		ifTrue: [ 
			propertyNamesAsSymbols 
				ifTrue: [ name asSymbol ]
				ifFalse: [ name ] ]
		ifFalse: [ 
			self error: 'property name must be a string, not ', name asString ]!   
encodeKey: key value: value
	self nextPut: key.
	self prettyPrintSpace.
	writeStream nextPut: $:.
	self prettyPrintSpace.
	self nextPut: value!  
indentedDo: block
	level := level + 1.
	block value.
	level := level - 1!  
printOn: stream
	super printOn: stream.
	stream nextPut: $(; print: subjectClass; nextPut: $)!  
neoJsonOn: neoJSONWriter
	neoJSONWriter writeFloat: self asFloat!
listClass
	"Return the collection type to use for JSON lists"
	
	^ listClass ifNil: [ Array ]! 
mapElementSeparator
	writeStream nextPut: $,.
	self newlineIndent
!
neoJsonOn: neoJSONWriter
	neoJSONWriter writeBoolean: self!  
neoJsonOn: neoJSONWriter
	neoJSONWriter writeInteger: self!  
writeBoolean: boolean
	boolean printOn: writeStream! 
writeListStreamingDo: block
	writeStream nextPut: $[.
	self writeStreamingDo: block.
	writeStream nextPut: $]! 
on: readStream
	"Initialize on readStream, which should be a character stream that 
	implements #next, #peek, #atEnd and (optionally) #close."

	^ self new
		on: readStream;
		yourself!   
parseCharacterHexDigit
	| digit |
	readStream atEnd 
		ifFalse: [ 
			digit := readStream next asInteger.
			(digit between: "$0" 48 and: "$9" 57)
				ifTrue: [ ^ digit - 48 ].
			(digit between: "$A" 65 and: "$F" 70)
				ifTrue: [ ^ digit - 55 ].
			(digit between: "$a" 97 and: "$f" 102)
				ifTrue: [ ^ digit - 87 ] ].
	self error: 'hex-digit expected'!   
initialize 
	super initialize.
	propertyNamesAsSymbols := false!
on: aReadStream
	"Initialize on aReadStream, which should be a character stream that 
	implements #next, #peek, #atEnd and (optionally) #close."
	
	readStream := aReadStream!
parseNumber
	| negated number |
	(negated := readStream peek = $-)
		ifTrue: [ readStream next ].
	number := self parseNumberInteger.
	(readStream peek = $.) 
		ifTrue: [
			readStream next. 
			number := number + self parseNumberFraction ].
	(readStream peek = $e or: [ readStream peek = $E ]) 
		ifTrue: [
			readStream next.
			number := number * self parseNumberExponent ].
	negated 
		ifTrue: [ number := number negated ].
	self consumeWhitespace.
	^ number!   
valueSchema: schemaName
	valueSchema := schemaName!  
writeObject: anObject on: jsonMapWriter
	| value |
	value := getter value: anObject.
	value 
		ifNotNil: [ jsonMapWriter writeKey: propertyName value: value ] !  
mapInstVars
	self mapInstVars: subjectClass instVarNames!
parseMapDo: block
	self expectChar: ${.
	(self matchChar: $})
		ifTrue: [ ^ self ].
	[ readStream atEnd ] whileFalse: [
		block value.
		(self matchChar: $}) 
			ifTrue: [ ^ self ].
		self expectChar: $, ].
	self error: 'end of map expected'!   
consumeWhitespace
	"Strip whitespaces from the input stream."

	[ readStream atEnd not and: [ readStream peek isSeparator ] ]
		whileTrue: [ 
			readStream next ]!  
new

	^super new
		initialize;
		yourself.!   
prettyPrintSpace
	prettyPrint
		ifTrue: [ writeStream space ]
!
toStringPretty: object
	^ String streamContents: [ :stream |
			(self on: stream)
				prettyPrint: true; 
				nextPut: object ]!  
on: jsonWriter
	^ self new
		on: jsonWriter;
		yourself!   
neoJsonOn: neoJSONWriter
	neoJSONWriter writeList: self! 
neoJsonOn: neoJSONWriter
	neoJSONWriter writeNull!   
close
	writeStream ifNotNil: [
		writeStream close.
		writeStream := nil ]!
printOn: stream
	super printOn: stream.
	stream nextPut: $(; print: schemaName; nextPut: $)!
parseNumberExponent
	| number negated |
	number := 0.
	negated := readStream peek = $-.
	(negated or: [ readStream peek = $+ ]) 
		ifTrue: [ readStream next ].
	[ readStream atEnd not and: [ readStream peek isDigit ] ] 
		whileTrue: [ 
			number := 10 * number + readStream next digitValue ].
	negated 
		ifTrue: [ number := number negated ].
	^ 10 raisedTo: number! 
encodeChar: char
	| code |
	code := char codePoint.
	(code < 32 or: [ code = 34 or: [ code = 92 ] ])
		ifTrue: [ self escapeChar: code ]
		ifFalse: [ writeStream nextPut: char ]!   
listOfType: collectionClass
	"Set the reader to a block that will create an instance of collectionClass 
	using #streamContents: while reading a list of elements"

	self reader: [ :jsonReader |
		collectionClass streamContents: [ :stream |
			jsonReader parseListDo: [ | element |
				element := jsonReader next.
				stream nextPut: element ] ] ]
	"we do not set a writer, #neoJsonOn: should be sufficient"
!
listClass: collectionClass
	listClass := collectionClass!
encoder: block
	"Set a encoder block that accepts the next object to be written 
	and converts it into the actual representation to be written."

	self writer: [ :jsonWriter :anObject | | representation |
		representation := block value: anObject.
		jsonWriter nextPut: representation ]! 
listOfElementSchema: elementSchema
	"Set the reader to a block that will create an instance of the
	standard listClass while reading a list of elements of type elementSchema"

	self reader: [ :jsonReader |
		jsonReader listClass streamContents: [ :stream |
			jsonReader parseListDo: [ | element |
				element := jsonReader nextAs: elementSchema.
				stream nextPut: element ] ] ]
	"we do not set a writer, #neoJsonOn: should be sufficient"
!  
listOfType: collectionClass andElementSchema: elementSchema
	"Set the reader to a block that will create an instance of collectionClass 
	using #streamContents: while reading a list of elements of type elementSchema"

	self reader: [ :jsonReader |
		collectionClass streamContents: [ :stream |
			jsonReader parseListDo: [ | element |
				element := jsonReader nextAs: elementSchema.
				stream nextPut: element ] ] ]
	"we do not set a writer, #neoJsonOn: should be sufficient"
! 
decoder: block
	"Set a decoder block that accepts the next object representation read 
	and converts it into the actual object to be returned."

	self reader: [ :jsonReader | | representation |
		representation := jsonReader next.
		block value: representation ]! 
neoJsonOn: neoJSONWriter
	neoJSONWriter writeObject: self!   
mappingFor: schema
	^ self 
		mappingFor: schema 
		ifAbsent: [ NeoJSONMappingNotFound signalFor: schema in: self ]!   
neoJsonOn: neoJSONWriter
	neoJSONWriter writeMap: self!  
writeMap: keyValueCollection
	self writeMapStreamingDo: [ :jsonMapWriter |
		keyValueCollection keysAndValuesDo: [ :key :value |
			jsonMapWriter writeKey: key value: value ] ]!  
getter: block
	getter := block!  
writer: block
	"Set the writer to block, a 2 argument block that when given: a jsonWriter 
	and the object to write will write the object that I represent."
	
	writer := block!  
propertyNamed: propertyName ifAbsent: block
	^ properties at: propertyName ifAbsent: block!  
writeObject: anObject on: jsonWriter
	jsonWriter writeMapStreamingDo: [ :jsonMapWriter |
		properties valuesDo: [ :each |
			each writeObject: anObject on: jsonMapWriter ] ]! 
writeString: string
	writeStream nextPut: $".
	string do: [ :each |
		self encodeChar: each ].
	writeStream nextPut: $"!