

Object
	subclass: #NeoJSONCustomMapping
	instanceVariableNames: 'schemaName reader writer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SPPL - Neo - JSON'!

!NeoJSONCustomMapping class methodsFor: 'comments' !
   
CLASSCOMMENTS
"I am NeoJSONCustomMapping.

I read the next JSON value and turn it into a Smalltalk object using a converter block. "! !
  
Object
	subclass: #NeoJSONMapper
	instanceVariableNames: 'mappings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SPPL - Neo - JSON'!

!NeoJSONMapper class methodsFor: 'comments' !
 
CLASSCOMMENTS
"I am NeoJSONMapper.
I hold a mapping from schema identifiers to objects that known how to read or write instances from or to JSON.

Schema identifiers are either Smalltalk classes or Symbols.

I offer a builder interface to construct mappings. 

The standard object mapping represents a Smalltalk object as a map, mapping some or all instance variables, or a set of accessors (getter/setter methods). Furthermore, for each value an optional schema can be specified.

Using a custom mapping, a reader and/or writer block can deal with special cases such as specific collection types with an optional schema for the elements, or a direct mapping of semi primitive types such as Date or DateAndTime.

A mapping can be specified explicitely on a mapper, or can be resolved using the #neoJsonMapping: class method.

Here are some examples:

	mapper mapAllInstVarsFor: Point.
	mapper for: TestObject do: [ :mapping |
		mapping mapInstVars: #(id name).
		(mapping mapInstVar: #timestamp to: 'created-at') valueSchema: DateAndTime.
		(mapping mapInstVar: #points) valueSchema: #ArrayOfPoints.
		(mapping mapInstVar: #bytes) valueSchema: ByteArray ].
	mapper for: DateAndTime customDo: [ :mapping |
		mapping decoder: [ :string | DateAndTime fromString: string ].
		mapping encoder: [ :dateAndTime | dateAndTime printString ] ].
	mapper for: #ArrayOfPoints customDo: [ :mapping |
		mapping listOfElementSchema: Point ].  
	mapper for: ByteArray customDo: [ :mapping |
		mapping listOfType: ByteArray ]
 "! !
  
NotFound
	subclass: #NeoJSONMappingNotFound
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SPPL - Neo - JSON'!

!NeoJSONMappingNotFound class methodsFor: 'comments' !
 
CLASSCOMMENTS
"I am NeoJSONMappingNotFound.
I am a NotFound exception.

I am signalled when no mapping is found for a schema. "! !
  
Object
	subclass: #NeoJSONObjectMapping
	instanceVariableNames: 'subjectClass properties'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SPPL - Neo - JSON'!

!NeoJSONObjectMapping class methodsFor: 'comments' !

CLASSCOMMENTS
"I am NeoJSONObjectMapping.
I describe how instances of a Smalltalk class are mapped to a JSON map.

I do this using set of NeoJSONPropertyMappings.

I know the Smalltalk class that I am a mapping for. This class is the schema identifier and is used to instanciate new objects.

A mapping can be based on any combination of instance variables or accessors (getter/setter pairs). 

I offer a builder interface to constuct property mappings. "! !

Error
	subclass: #NeoJSONParseError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SPPL - Neo - JSON'!

!NeoJSONParseError class methodsFor: 'comments' !
  
CLASSCOMMENTS
"I am NeoJSONParseError.
I am an Error.

I am signalled when JSON input is not valid while parsing. "! !
  
Object
	subclass: #NeoJSONPropertyMapping
	instanceVariableNames: 'propertyName valueSchema getter setter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SPPL - Neo - JSON'!

!NeoJSONPropertyMapping class methodsFor: 'comments' !
 
CLASSCOMMENTS
"I am NeoJSONPropertyMapping.
I describe how a property of a Smalltalk class maps to a JSON property.

A property is either an instance variable or getter/setter pair.
I hold a getter and setter block to do the actual work. The property name is used in JSON.

Optionally, the schema for the value of my property can be specified. This is then used recursively. "! !
  
NeoJSONMapper
	subclass: #NeoJSONReader
	instanceVariableNames: 'readStream stringStream listClass mapClass propertyNamesAsSymbols'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SPPL - Neo - JSON'!

!NeoJSONReader class methodsFor: 'comments' !
 
CLASSCOMMENTS
"I am NeoJSONReader.
I am a NeoJSONMapper.
I parse a JSON representation from a textual stream into Smalltalk objects.

Primitive types are automatically mapped to their Smalltalk counterpart: Numbers, Strings, Booleans and UndefinedObject.
When parsing using #next, lists normally become Arrays and maps become Dictionaries.
An alternative listClass or mapClass can be specified.

When parsing using #nextAs: a schema can be specified. This schema defines a mapping to be used.

Here are some examples:

	NeoJSONReader fromString: ' [ 1,2,3 ] '.
	NeoJSONReader fromString: ' [ 3.14159, true, false, null, ""string"" ] '.
	NeoJSONReader fromString: ' { ""x"" : 1, ""y"" : 2 } '.
	
	(NeoJSONReader on: ' { ""x"" : 1, ""y"" : 2 } ' readStream)
		mapInstVarsFor: Point;
		nextAs: Point.	
			
	(NeoJSONReader on: '[ { ""x"" : 1, ""y"" : 2 }, { ""x"" : 3, ""y"" : 4 } ]' readStream)
		mapInstVarsFor: Point;
		for: #ArrayOfPoints customDo: [ :mapping |
			mapping listOfElementSchema: Point ];
		nextAs: #ArrayOfPoints. "! !
  
Object
	subclass: #NeoJSONStreamingWriter
	instanceVariableNames: 'jsonWriter first'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SPPL - Neo - JSON'!

!NeoJSONStreamingWriter class methodsFor: 'comments' !
   
CLASSCOMMENTS
"I am NeoJSONStreamingWriter.
I help in writing streamed maps and lists.

This is an internal class used by NeoJSONWriter to deal with interleaving elements and separators as well as pretty printing. "! !
  
NeoJSONMapper
	subclass: #NeoJSONWriter
	instanceVariableNames: 'writeStream prettyPrint level'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SPPL - Neo - JSON'!

!NeoJSONWriter class methodsFor: 'comments' !
 
CLASSCOMMENTS
"I am NeoJSONWriter.
I am a NeoJSONMapper.
I write a JSON representation of Smalltalk objects to a textual stream.

Objects implementing #neoJsonOn: can be encoded.
A number of primitive types, like Strings, Numbers, Booleans and UndefinedObject are treated specially.
Collection are encoded as lists, except for Dictionaries that are encoded as maps.

All other objects need a mapping to help in doing the encoding.

Here are some examples:

	NeoJSONWriter toString: #(1 2 3).
	NeoJSONWriter toString: { Float pi. true. false. 'string' }.
	NeoJSONWriter toStringPretty: (Dictionary new at: #x put: 1; at: #y put: 2; yourself).
	
	String streamContents: [ :stream |
		(NeoJSONWriter on: stream)
			prettyPrint: true;
			mapInstVarsFor: Point;
			nextPut: (Array with: 1@3 with: -1@3) ].
	 "! !

!NeoJSONCustomMapping methodsFor: 'mapping' !
  
decoder: block
	"Set a decoder block that accepts the next object representation read 
	and converts it into the actual object to be returned."

	self reader: [ :jsonReader | | representation |
		representation := jsonReader next.
		block value: representation ]!
   
encoder: block
	"Set a encoder block that accepts the next object to be written 
	and converts it into the actual representation to be written."

	self writer: [ :jsonWriter :anObject | | representation |
		representation := block value: anObject.
		jsonWriter nextPut: representation ]!
   
listOfElementSchema: elementSchema
	"Set the reader to a block that will create an instance of the
	standard listClass while reading a list of elements of type elementSchema"

	self reader: [ :jsonReader |
		jsonReader listClass streamContents: [ :stream |
			jsonReader parseListDo: [ | element |
				element := jsonReader nextAs: elementSchema.
				stream nextPut: element ] ] ]
	"we do not set a writer, #neoJsonOn: should be sufficient"
!

listOfType: collectionClass
	"Set the reader to a block that will create an instance of collectionClass 
	using #streamContents: while reading a list of elements"

	self reader: [ :jsonReader |
		collectionClass streamContents: [ :stream |
			jsonReader parseListDo: [ | element |
				element := jsonReader next.
				stream nextPut: element ] ] ]
	"we do not set a writer, #neoJsonOn: should be sufficient"
!
  
listOfType: collectionClass andElementSchema: elementSchema
	"Set the reader to a block that will create an instance of collectionClass 
	using #streamContents: while reading a list of elements of type elementSchema"

	self reader: [ :jsonReader |
		collectionClass streamContents: [ :stream |
			jsonReader parseListDo: [ | element |
				element := jsonReader nextAs: elementSchema.
				stream nextPut: element ] ] ]
	"we do not set a writer, #neoJsonOn: should be sufficient"
! !

!NeoJSONCustomMapping methodsFor: 'accessing' !
  
reader: block
	"Set the reader to block, a 1 argument block that when given a jsonReader 
	as argument reads the object that I represent"
	
	reader := block!
   
schemaName: schema
	schemaName := schema!
  
writer: block
	"Set the writer to block, a 2 argument block that when given: a jsonWriter 
	and the object to write will write the object that I represent."
	
	writer := block! !

!NeoJSONCustomMapping methodsFor: 'writing' !
 
writeObject: anObject on: jsonWriter
	writer value: jsonWriter value: anObject! !

!NeoJSONCustomMapping methodsFor: 'parsing' !
 
readFrom: jsonReader
	^ reader value: jsonReader! !

!NeoJSONCustomMapping methodsFor: 'printing' !
  
printOn: stream
	super printOn: stream.
	stream nextPut: $(; print: schemaName; nextPut: $)! !

!NeoJSONMapper methodsFor: 'mapping' !
  
for: schemaName customDo: block
	"Create and add a new custom mapping for schemaName.
	The code in block should further customize the mapping."
	
	| mapping |
	mapping := self newCustomMappingFor: schemaName.
	block value: mapping.
	^ mapping!
  
for: smalltalkClass do: block
	"Create and add a new standard object mapping for smalltalkClass.
	The code in block should further customize the mapping."
	
	| mapping |
	mapping := self newObjectMappingFor: smalltalkClass.
	block value: mapping.
	^ mapping! !

!NeoJSONMapper methodsFor: 'accessing' !
 
mappingFor: schema
	^ self 
		mappingFor: schema 
		ifAbsent: [ NeoJSONMappingNotFound signalFor: schema in: self ]!
 
mappingFor: schema ifAbsent: block
	(self mappings includesKey: schema)
		ifFalse: [ self resolveMappingFor: schema ].
	^ self mappings
		at: schema
		ifAbsent: block! !

!NeoJSONMapper methodsFor: 'private' !

mappings
	^ mappings
		ifNil: [ mappings := Dictionary new ]!
 
newCustomMappingFor: schemaName
	| mapping |
	mapping := NeoJSONCustomMapping new.
	mapping schemaName: schemaName.
	self mappings at: schemaName put: mapping.
	^ mapping!

newObjectMappingFor: smalltalkClass
	| mapping |
	mapping := NeoJSONObjectMapping new.
	mapping subjectClass: smalltalkClass.
	self mappings at: smalltalkClass put: mapping.
	^ mapping!
  
resolveMappingFor: class
	(class respondsTo: #neoJsonMapping:)
		ifTrue: [ 
			class perform: #neoJsonMapping: with: self ]! !

!NeoJSONMapper methodsFor: 'convencience' !

mapAllInstVarsFor: smalltalkClass
	"Map the instance variables of smalltalkClass and all its super classes"

	| mapping |
	mapping := self newObjectMappingFor: smalltalkClass.
	mapping mapAllInstVars.
	^ mapping!
  
mapInstVarsFor: smalltalkClass
	"Map the instance variables of smalltalkClass"

	| mapping |
	mapping := self newObjectMappingFor: smalltalkClass.
	mapping mapInstVars.
	^ mapping! !

!NeoJSONMappingNotFound methodsFor: 'printing' !

standardMessageText
	^ String streamContents: [ :stream |
		stream << 'No mapping found for '.
		stream print: self object.
		stream << ' in '.
		stream print: self collection class ]! !

!NeoJSONObjectMapping methodsFor: 'writing' !

writeObject: anObject on: jsonWriter
	jsonWriter writeMapStreamingDo: [ :jsonMapWriter |
		properties valuesDo: [ :each |
			each writeObject: anObject on: jsonMapWriter ] ]! !

!NeoJSONObjectMapping methodsFor: 'initialize-release' !
 
initialize
	super initialize.
	properties := Dictionary new.! !

!NeoJSONObjectMapping methodsFor: 'mapping' !
  
mapAccessor: accessor
	^ self mapAccessor: accessor to: accessor !
 
mapAccessor: accessor mutator: mutator to: propertyName
	^ self 
		mapProperty: propertyName
		getter: [ :object | object perform: accessor ]
		setter: [ :object :value | object perform: mutator with: value ] !
  
mapAccessor: accessor to: propertyName
	^ self 
		mapProperty: propertyName
		getter: [ :object | object perform: accessor ]
		setter: [ :object :value | object perform: accessor asMutator with: value ] !

mapAccessors: accessors
	accessors do: [ :each |
		self mapAccessor: each ]!
  
mapInstVar: instVarName
	^ self mapInstVar: instVarName to: instVarName !
  
mapInstVar: instVarName to: propertyName
	| index |
	index := subjectClass 
		instVarIndexFor: instVarName 
		ifAbsent: [ self error: 'Inst var not found: ', instVarName printString ].
	^ self 
		mapProperty: propertyName
		getter: [ :object | object instVarAt: index ]
		setter: [ :object :value | object instVarAt: index put: value ] !
   
mapInstVars: instVarNames
	instVarNames do: [ :each |
		self mapInstVar: each ]!
  
mapProperty: propertyName getter: readBlock setter: writeBlock
	| propertyMapping |
	propertyMapping := NeoJSONPropertyMapping new.
	propertyMapping 
		propertyName: propertyName;
		getter: readBlock;
		setter: writeBlock.
	properties at: propertyName put: propertyMapping.
	^ propertyMapping ! !

!NeoJSONObjectMapping methodsFor: 'accessing' !
 
propertyNamed: propertyName ifAbsent: block
	^ properties at: propertyName ifAbsent: block!

subjectClass: smalltalkClass
	subjectClass := smalltalkClass! !

!NeoJSONObjectMapping methodsFor: 'printing' !
  
printOn: stream
	super printOn: stream.
	stream nextPut: $(; print: subjectClass; nextPut: $)! !

!NeoJSONObjectMapping methodsFor: 'convenience' !
 
mapAllInstVars
	self mapInstVars: subjectClass allInstVarNames!

mapInstVars
	self mapInstVars: subjectClass instVarNames! !

!NeoJSONObjectMapping methodsFor: 'parsing' !
   
readFrom: jsonReader
	| anObject |
	anObject := subjectClass new.
	jsonReader parseMapKeysDo: [ :key |
		(self propertyNamed: key ifAbsent: [ nil ])
			ifNil: [
				"read, skip & ignore value"
				jsonReader next ]
			ifNotNil: [ :mapping |
				mapping readObject: anObject from: jsonReader ] ].
	^ anObject! !

!NeoJSONObjectMapping class methodsFor: 'as yet unclassified' !
 
new

	^super new
		initialize;
		yourself.! !

!NeoJSONPropertyMapping methodsFor: 'accessing' !
  
getter: block
	getter := block!

propertyName: string
	propertyName := string!
  
setter: block
	setter := block!

valueSchema: schemaName
	valueSchema := schemaName! !

!NeoJSONPropertyMapping methodsFor: 'writing' !
   
writeObject: anObject on: jsonMapWriter
	| value |
	value := getter value: anObject.
	value 
		ifNotNil: [ jsonMapWriter writeKey: propertyName value: value ] ! !

!NeoJSONPropertyMapping methodsFor: 'parsing' !
   
readObject: anObject from: jsonReader
	| value |
	value := jsonReader nextAs: valueSchema.
	setter value: anObject value: value! !

!NeoJSONPropertyMapping methodsFor: 'printing' !
   
printOn: stream
	super printOn: stream.
	stream nextPut: $(; print: propertyName; nextPut: $)! !

!NeoJSONReader methodsFor: 'accessing' !
  
listClass
	"Return the collection type to use for JSON lists"
	
	^ listClass ifNil: [ Array ]!
   
mapClass
	"Return the collection type to use for JSON maps"
	
	^ mapClass ifNil: [ Dictionary ]!
 
next
	"Primary interface to parse a JSON value.
	Return either primitives, a listClass or a mapClass instance."
	
	^ self 
		consumeWhitespace; 
		parseValue!

nextAs: schema
	"Secondary interface to parse JSON.
	Return objects according to schema."
	
	| mapping |
	schema ifNil: [ ^ self next ].
	self consumeWhitespace.
	mapping := self mappingFor: schema.
	^ mapping readFrom: self! !

!NeoJSONReader methodsFor: 'parsing' !
   
parseCharacter
	| char |
	(char := readStream next) = $\ 
		ifFalse: [ ^ char ].
	('/"\' includes: (char := readStream next))
		ifTrue: [ ^ char ].
	('bfnrt' includes: char)
		ifTrue: [ ^ self unescapeChar: char ].
	char = $u
		ifTrue: [ ^ self parseCharacterHex ].
	self error: 'invalid escape character \' , (String with: char)!

parseCharacterHex
	| value |
	value := self parseCharacterHexDigit.
	3 timesRepeat: [ 
		value := (value << 4) + self parseCharacterHexDigit ].
	^ Character codePoint: value!
 
parseCharacterHexDigit
	| digit |
	readStream atEnd 
		ifFalse: [ 
			digit := readStream next asInteger.
			(digit between: "$0" 48 and: "$9" 57)
				ifTrue: [ ^ digit - 48 ].
			(digit between: "$A" 65 and: "$F" 70)
				ifTrue: [ ^ digit - 55 ].
			(digit between: "$a" 97 and: "$f" 102)
				ifTrue: [ ^ digit - 87 ] ].
	self error: 'hex-digit expected'!
 
parseList
	^ self listClass 
		streamContents: [ :stream |
			self parseListElementsDo: [ :each |
				stream nextPut: each ] ]

		!
   
parseListDo: block
	self expectChar: $[.
	(self matchChar: $]) 
		ifTrue: [ ^ self ].
	[ readStream atEnd ] 
		whileFalse: [
			block value.
			(self matchChar: $]) 
				ifTrue: [ ^ self ].
			self expectChar: $, ].
	self error: 'end of list expected'!
  
parseListElementsDo: block
	self parseListDo: [ 
		block value: self parseValue ]!

parseMap
	| map |
	map := self mapClass new.
	self parseMapKeysAndValuesDo: [ :key :value |
		map at: key put: value ].
	^ map!

parseMapDo: block
	self expectChar: ${.
	(self matchChar: $})
		ifTrue: [ ^ self ].
	[ readStream atEnd ] whileFalse: [
		block value.
		(self matchChar: $}) 
			ifTrue: [ ^ self ].
		self expectChar: $, ].
	self error: 'end of map expected'!
 
parseMapKeysAndValuesDo: block
	self parseMapKeysDo: [ :key |
		block value: key value: self parseValue ]!

parseMapKeysDo: block
	self parseMapDo: [ | key |
		key := self parsePropertyName.
		self expectChar: $:.
		block value: key ]!
 
parseNumber
	| negated number |
	(negated := readStream peek = $-)
		ifTrue: [ readStream next ].
	number := self parseNumberInteger.
	(readStream peek = $.) 
		ifTrue: [
			readStream next. 
			number := number + self parseNumberFraction ].
	(readStream peek = $e or: [ readStream peek = $E ]) 
		ifTrue: [
			readStream next.
			number := number * self parseNumberExponent ].
	negated 
		ifTrue: [ number := number negated ].
	self consumeWhitespace.
	^ number!
 
parseNumberExponent
	| number negated |
	number := 0.
	negated := readStream peek = $-.
	(negated or: [ readStream peek = $+ ]) 
		ifTrue: [ readStream next ].
	[ readStream atEnd not and: [ readStream peek isDigit ] ] 
		whileTrue: [ 
			number := 10 * number + readStream next digitValue ].
	negated 
		ifTrue: [ number := number negated ].
	^ 10 raisedTo: number!
   
parseNumberFraction
	| number power |
	number := 0.
	power := 1.0.
	[ readStream atEnd not and: [ readStream peek isDigit ] ] 
		whileTrue: [
			number := 10 * number + readStream next digitValue.
			power := power * 10.0 ].
	^ number / power!
 
parseNumberInteger
	| number |
	number := nil.
	[ readStream atEnd not and: [ readStream peek isDigit ] ] 
		whileTrue: [ 
			number := 10 * (number ifNil: [ 0 ]) + readStream next digitValue ].
	number ifNil: [ self error: 'Integer digit expected' ].
	^ number!
   
parsePropertyName
	| name |
	name := self parseValue.
	^ name isString
		ifTrue: [ 
			propertyNamesAsSymbols 
				ifTrue: [ name asSymbol ]
				ifFalse: [ name ] ]
		ifFalse: [ 
			self error: 'property name must be a string, not ', name asString ]!
 
parseString
	| result |
	self expectChar: $".
	result := self stringStreamContents: [ :stream |
		[ readStream atEnd or: [ readStream peek = $"] ] 
			whileFalse: [ 
				stream nextPut: self parseCharacter ] ].
	self expectChar: $".
	^ result!
 
parseValue
	| char |
	readStream atEnd ifFalse: [ 
		char := readStream peek.
		char = ${
			ifTrue: [ ^ self parseMap ].
		char = $[
			ifTrue: [ ^ self parseList ].
		char = $"
			ifTrue: [ ^ self parseString ].
		(char = $- or: [ char isDigit ])
			ifTrue: [ ^ self parseNumber ].
		self parseConstantDo: [ :value | ^ value ] ].
	self error: 'invalid input: ', char asString! !

!NeoJSONReader methodsFor: 'private' !
 
consumeWhitespace
	"Strip whitespaces from the input stream."

	[ readStream atEnd not and: [ readStream peek isSeparator ] ]
		whileTrue: [ 
			readStream next ]!

error: text
	NeoJSONParseError signal: text!
   
expectChar: character
	"Expect character and consume input and optional whitespace at the end,
	 throw an error otherwise."

	(self matchChar: character)
		ifFalse: [ self error: character asString, ' expected' ]!
  
match: string do: block
	"Try to read and consume string and execute block if successful.
	Else do nothing (but do not back up)"
	 
	(string allSatisfy: [ :each | self matchChar: each ])
		ifTrue: block
!
  
matchChar: character
	"Tries to match character, consume input and 
	answer true if successful and consumes whitespace at the end.
	When character does not match, return false and leave stream untouched."
	
	^ readStream peek = character
		ifTrue: [
			readStream next.
			self consumeWhitespace.
			true ]
		ifFalse: [
			false ]!
  
parseConstantDo: block
	"Parse and consume either true|false|nil|null and execute block 
	or else do nothing (but do not back up).
	Hand written implementation to avoid the use of #position:"
	
	(readStream peek = $t)
		ifTrue: [
			^ self match: 'true' do: [ block value: true ] ].
	(readStream peek = $f)
		ifTrue: [
			^ self match: 'false' do: [ block value: false ] ].
	(readStream peek = $n)
		ifTrue: [
			^ self match: 'null' do: [ block value: nil ] ]
!

stringStreamContents: block
	"Like String streamContents: block
	but reusing the underlying buffer for improved efficiency"
	
	stringStream 
		ifNil: [ 
			stringStream := (String new: 32) writeStream ].
	stringStream reset.
	block value: stringStream.
	^ stringStream contents!
 
unescapeChar: char
	"Handle one of the named character escapes, b, f, n, r or t. Else fail."
	
	char = $b
		ifTrue: [ ^ Character backspace ].
	char = $f
		ifTrue: [ ^ Character newPage ].
	char = $n
		ifTrue: [ ^ Character lf ].
	char = $r
		ifTrue: [ ^ Character cr ].
	char = $t
		ifTrue: [ ^ Character tab ].
	self error: 'Unknown escape char: ', (String with: char)! !

!NeoJSONReader methodsFor: 'initialize-release' !
 
close
	readStream ifNotNil: [
		readStream close.
		readStream := nil ]!
 
initialize 
	super initialize.
	propertyNamesAsSymbols := false!
  
listClass: collectionClass
	listClass := collectionClass!
  
mapClass: collectionClass
	mapClass := collectionClass!

on: aReadStream
	"Initialize on aReadStream, which should be a character stream that 
	implements #next, #peek, #atEnd and (optionally) #close."
	
	readStream := aReadStream!
  
propertyNamesAsSymbols: boolean
	propertyNamesAsSymbols := boolean! !

!NeoJSONReader class methodsFor: 'instance creation' !

on: readStream
	"Initialize on readStream, which should be a character stream that 
	implements #next, #peek, #atEnd and (optionally) #close."

	^ self new
		on: readStream;
		yourself! !

!NeoJSONReader class methodsFor: 'convenience' !
   
fromString: string
	^ (self on: string readStream) next!
   
fromString: string as: schema
	^ (self on: string readStream) nextAs: schema!
  
new

	^super new
		initialize;
		yourself.! !

!NeoJSONStreamingWriter methodsFor: 'testing' !

wasUsed
	^ first not! !

!NeoJSONStreamingWriter methodsFor: 'writing' !
 
writeElement: value
	first 
		ifTrue: [ 
			jsonWriter newlineIndent. 
			first := false ]
		ifFalse: [ 
			jsonWriter listElementSeparator ].
	jsonWriter nextPut: value !
  
writeKey: key value: value
	first 
		ifTrue: [ 
			jsonWriter newlineIndent. 
			first := false ]
		ifFalse: [ 
			jsonWriter mapElementSeparator ].
	jsonWriter encodeKey: key value: value ! !

!NeoJSONStreamingWriter methodsFor: 'initialize-release' !
   
initialize
	super initialize.
	first := true!
 
on: aJSONWriter
	jsonWriter := aJSONWriter! !

!NeoJSONStreamingWriter class methodsFor: 'instance creation' !
   
new

	^super new
		initialize;
		yourself.!
 
on: jsonWriter
	^ self new
		on: jsonWriter;
		yourself! !

!NeoJSONWriter methodsFor: 'accessing' !
   
nextPut: anObject
	anObject neoJsonOn: self! !

!NeoJSONWriter methodsFor: 'writing' !
   
writeBoolean: boolean
	boolean printOn: writeStream!
   
writeFloat: float
	float printOn: writeStream!
 
writeInteger: integer
	integer printOn: writeStream
	!

writeList: collection
	self writeListStreamingDo: [ :jsonListWriter |
		collection do: [ :each |
			jsonListWriter writeElement: each ] ]!
   
writeListStreamingDo: block
	writeStream nextPut: $[.
	self writeStreamingDo: block.
	writeStream nextPut: $]!
   
writeMap: keyValueCollection
	self writeMapStreamingDo: [ :jsonMapWriter |
		keyValueCollection keysAndValuesDo: [ :key :value |
			jsonMapWriter writeKey: key value: value ] ]!

writeMapStreamingDo: block
	writeStream nextPut: ${.
	self writeStreamingDo: block.
	writeStream nextPut: $}!

writeNull
	writeStream nextPutAll: 'null'!
 
writeObject: anObject
	| mapping |
	mapping := self mappingFor: anObject class.
	mapping
		writeObject: anObject
		on: self!
   
writeStreamingDo: block
	| mapWriter |
	mapWriter := NeoJSONStreamingWriter on: self.
	self indentedDo: [
		block value: mapWriter ].
	mapWriter wasUsed
		ifTrue: [ self newlineIndent ]
		ifFalse: [ self prettyPrintSpace ]!
  
writeString: string
	writeStream nextPut: $".
	string do: [ :each |
		self encodeChar: each ].
	writeStream nextPut: $"! !

!NeoJSONWriter methodsFor: 'private' !

encodeChar: char
	| code |
	code := char codePoint.
	(code < 32 or: [ code = 34 or: [ code = 92 ] ])
		ifTrue: [ self escapeChar: code ]
		ifFalse: [ writeStream nextPut: char ]!
 
encodeKey: key value: value
	self nextPut: key.
	self prettyPrintSpace.
	writeStream nextPut: $:.
	self prettyPrintSpace.
	self nextPut: value!

escapeChar: code
	code = 34
		ifTrue: [ ^ writeStream nextPutAll: '\"' ].
	code = 92
		ifTrue: [ ^ writeStream nextPutAll: '\\' ].
	code = 47
		ifTrue: [ ^ writeStream nextPutAll: '\/' ].
	code = 8
		ifTrue: [ ^ writeStream nextPutAll: '\b' ].
	code = 12
		ifTrue: [ ^ writeStream nextPutAll: '\f' ].		
	code = 10
		ifTrue: [ ^ writeStream nextPutAll: '\n' ].		
	code = 13
		ifTrue: [ ^ writeStream nextPutAll: '\r' ].		
	code = 9
		ifTrue: [ ^ writeStream nextPutAll: '\t' ].		
	writeStream nextPutAll: '\u'.
	code printOn: writeStream base: 16 nDigits: 4!
 
indentedDo: block
	level := level + 1.
	block value.
	level := level - 1!

listElementSeparator
	writeStream nextPut: $,.
	self newlineIndent
!
 
mapElementSeparator
	writeStream nextPut: $,.
	self newlineIndent
!
  
newlineIndent
	prettyPrint
		ifTrue: [ 
			writeStream nextPut: Character cr.
			level timesRepeat: [ writeStream nextPut: Character tab ] ]!
   
prettyPrintSpace
	prettyPrint
		ifTrue: [ writeStream space ]
! !

!NeoJSONWriter methodsFor: 'initialize-release' !
   
close
	writeStream ifNotNil: [
		writeStream close.
		writeStream := nil ]!
  
initialize
	super initialize.
	prettyPrint := false.
	level := 0!

on: aWriteStream
	"Initialize on aWriteStream, which should be a character stream that 
	implements #nextPut:, #nextPutAll:, #space and (optionally) #close."

	writeStream := aWriteStream
!
  
prettyPrint: boolean
	prettyPrint := boolean! !

!NeoJSONWriter class methodsFor: 'instance creation' !
  
on: writeStream
	"Initialize on writeStream, which should be a character stream that 
	implements #nextPut:, #nextPutAll:, #space and (optionally) #close."

	^ self new
		on: writeStream;
		yourself! !

!NeoJSONWriter class methodsFor: 'convenience' !
 
new

	^super new
		initialize;
		yourself.!
 
toString: object
	^ String streamContents: [ :stream |
			(self on: stream) nextPut: object ]!

toStringPretty: object
	^ String streamContents: [ :stream |
			(self on: stream)
				prettyPrint: true; 
				nextPut: object ]! !

!Boolean methodsFor: '*neo-json-core' !
   
neoJsonOn: neoJSONWriter
	neoJSONWriter writeBoolean: self! !

!Collection methodsFor: '*neo-json-core' !

neoJsonOn: neoJSONWriter
	neoJSONWriter writeList: self! !

!Dictionary methodsFor: '*neo-json-core' !
   
neoJsonOn: neoJSONWriter
	neoJSONWriter writeMap: self! !

!Integer methodsFor: '*neo-json-core' !
   
neoJsonOn: neoJSONWriter
	neoJSONWriter writeInteger: self! !

!Number methodsFor: '*neo-json-core' !

neoJsonOn: neoJSONWriter
	neoJSONWriter writeFloat: self asFloat! !

!Object methodsFor: '*neo-json-core' !
  
neoJsonOn: neoJSONWriter
	neoJSONWriter writeObject: self! !

!String methodsFor: '*neo-json-core' !
 
neoJsonOn: neoJSONWriter
	neoJSONWriter writeString: self! !

!UndefinedObject methodsFor: '*neo-json-core' !

neoJsonOn: neoJSONWriter
	neoJSONWriter writeNull! !
   
NeoJSONMappingNotFound comment: '"I am NeoJSONMappingNotFound.
I am a NotFound exception.

I am signalled when no mapping is found for a schema. "'!
 
NeoJSONWriter comment: '"I am NeoJSONWriter.
I am a NeoJSONMapper.
I write a JSON representation of Smalltalk objects to a textual stream.

Objects implementing #neoJsonOn: can be encoded.
A number of primitive types, like Strings, Numbers, Booleans and UndefinedObject are treated specially.
Collection are encoded as lists, except for Dictionaries that are encoded as maps.

All other objects need a mapping to help in doing the encoding.

Here are some examples:

	NeoJSONWriter toString: #(1 2 3).
	NeoJSONWriter toString: { Float pi. true. false. ''string'' }.
	NeoJSONWriter toStringPretty: (Dictionary new at: #x put: 1; at: #y put: 2; yourself).
	
	String streamContents: [ :stream |
		(NeoJSONWriter on: stream)
			prettyPrint: true;
			mapInstVarsFor: Point;
			nextPut: (Array with: 1@3 with: -1@3) ].
	 "'!
 
NeoJSONMapper comment: '"I am NeoJSONMapper.
I hold a mapping from schema identifiers to objects that known how to read or write instances from or to JSON.

Schema identifiers are either Smalltalk classes or Symbols.

I offer a builder interface to construct mappings. 

The standard object mapping represents a Smalltalk object as a map, mapping some or all instance variables, or a set of accessors (getter/setter methods). Furthermore, for each value an optional schema can be specified.

Using a custom mapping, a reader and/or writer block can deal with special cases such as specific collection types with an optional schema for the elements, or a direct mapping of semi primitive types such as Date or DateAndTime.

A mapping can be specified explicitely on a mapper, or can be resolved using the #neoJsonMapping: class method.

Here are some examples:

	mapper mapAllInstVarsFor: Point.
	mapper for: TestObject do: [ :mapping |
		mapping mapInstVars: #(id name).
		(mapping mapInstVar: #timestamp to: ''created-at'') valueSchema: DateAndTime.
		(mapping mapInstVar: #points) valueSchema: #ArrayOfPoints.
		(mapping mapInstVar: #bytes) valueSchema: ByteArray ].
	mapper for: DateAndTime customDo: [ :mapping |
		mapping decoder: [ :string | DateAndTime fromString: string ].
		mapping encoder: [ :dateAndTime | dateAndTime printString ] ].
	mapper for: #ArrayOfPoints customDo: [ :mapping |
		mapping listOfElementSchema: Point ].  
	mapper for: ByteArray customDo: [ :mapping |
		mapping listOfType: ByteArray ]
 "'!

NeoJSONCustomMapping comment: '"I am NeoJSONCustomMapping.

I read the next JSON value and turn it into a Smalltalk object using a converter block. "'!
   
NeoJSONObjectMapping comment: '"I am NeoJSONObjectMapping.
I describe how instances of a Smalltalk class are mapped to a JSON map.

I do this using set of NeoJSONPropertyMappings.

I know the Smalltalk class that I am a mapping for. This class is the schema identifier and is used to instanciate new objects.

A mapping can be based on any combination of instance variables or accessors (getter/setter pairs). 

I offer a builder interface to constuct property mappings. "'!
 
NeoJSONParseError comment: '"I am NeoJSONParseError.
I am an Error.

I am signalled when JSON input is not valid while parsing. "'!
  
NeoJSONPropertyMapping comment: '"I am NeoJSONPropertyMapping.
I describe how a property of a Smalltalk class maps to a JSON property.

A property is either an instance variable or getter/setter pair.
I hold a getter and setter block to do the actual work. The property name is used in JSON.

Optionally, the schema for the value of my property can be specified. This is then used recursively. "'!
 
NeoJSONReader comment: '"I am NeoJSONReader.
I am a NeoJSONMapper.
I parse a JSON representation from a textual stream into Smalltalk objects.

Primitive types are automatically mapped to their Smalltalk counterpart: Numbers, Strings, Booleans and UndefinedObject.
When parsing using #next, lists normally become Arrays and maps become Dictionaries.
An alternative listClass or mapClass can be specified.

When parsing using #nextAs: a schema can be specified. This schema defines a mapping to be used.

Here are some examples:

	NeoJSONReader fromString: '' [ 1,2,3 ] ''.
	NeoJSONReader fromString: '' [ 3.14159, true, false, null, ""string"" ] ''.
	NeoJSONReader fromString: '' { ""x"" : 1, ""y"" : 2 } ''.
	
	(NeoJSONReader on: '' { ""x"" : 1, ""y"" : 2 } '' readStream)
		mapInstVarsFor: Point;
		nextAs: Point.	
			
	(NeoJSONReader on: ''[ { ""x"" : 1, ""y"" : 2 }, { ""x"" : 3, ""y"" : 4 } ]'' readStream)
		mapInstVarsFor: Point;
		for: #ArrayOfPoints customDo: [ :mapping |
			mapping listOfElementSchema: Point ];
		nextAs: #ArrayOfPoints. "'!

NeoJSONStreamingWriter comment: '"I am NeoJSONStreamingWriter.
I help in writing streamed maps and lists.

This is an internal class used by NeoJSONWriter to deal with interleaving elements and separators as well as pretty printing. "'!